use super::*;

static STRING_TABLE: std::sync::LazyLock<Mutex<BTreeSet<&'static str>>> =
    std::sync::LazyLock::new(|| Mutex::new(BTreeSet::new()));

pub trait ToStaticStr {
    fn to_static_str(&self) -> &'static str;
}

impl<T: AsRef<str>> ToStaticStr for T {
    fn to_static_str(&self) -> &'static str {
        let s = self.as_ref();
        let mut string_table = STRING_TABLE.lock();
        if let Some(v) = string_table.get(s) {
            return v;
        }
        let ss = Box::leak(s.to_owned().into_boxed_str());
        string_table.insert(ss);
        ss
    }
}
